#ifndef CM_H
# 	define	CM_H
#       include "camdef.h"
#       include "camioctl.h"

/*
**  cam.h - to use only in CAMAC driver
**  hardware: sun4c/pti sun4e (sun4m)
*/

#define	reg_t	 /* from laziness & unix..... */

/* CES controller registers  structure				*/
typedef	struct	{
	ushort	qq0;
	ushort	ccsr;
	ushort	amd_data[6];
	ushort	qq1;
	ushort	ifr;
	ushort	amd_ctrl[6];
	ushort	qq2;
	ushort	car;
	ushort	qq3;
	ushort	btb_bz;
	ulong	gl;
}cm_t;

typedef	struct	{
	int	so_status;	/* status of slot		*/
	char   *so_ptr;         /* pointer in kern   buffer     */
	int	so_flags;
}cm_s_t;

typedef	struct {
	u_int   pa_kmx;         /* 16- pointer set              */
	caddr_t pa_ptr;
	int     pa_cnt;
}cm_p_t;

typedef	struct	{
	int	si_cminor;	/* device installing interrupt	*/
	int	si_pid;		/* pid of installing process	*/
	struct task_struct  
		*si_procp;  	/* task_struct ptr for send_sig */
}cm_i_t;

/* CIORES, CIOSHUT 						*/
typedef struct	{
	int	sr_status;	/* status of a crate		*/
	int	sr_crate;	/* crate			*/
	int	sr_func;	/* 0 restore 1 shut 		*/
}cm_r_t;

/****** hardware dependency *************************************/
/* physical base from which CES geographically addresses CAMAC 	*/
#ifdef	linux
#define	CM_VME_BASE	0x800000
#define VME_PAGESIZE	0x1000
#endif
#ifndef linux
#ifdef	sun4e
#define	CM_VME_BASE	0xFF800000
#endif
#ifdef	sun4c
#define	CM_VME_BASE	SB_WINDOW(CAMAC_DEVICE)
#endif	
#ifdef	sun4m
#ifdef  PTVME_DRIVER
#define CM_VME_BASE     0x800000
#else
#define	CM_VME_BASE	SB_WINDOW(CAMAC_DEVICE)
#endif
#endif
#endif
/****************************************************************/

struct  byte            { char abyte; };
struct  usword          { unsigned short  ushort; };
struct  uslong          { unsigned long   ulong; };

/* 
**	CAMAC hard addressing of cm_t control registers:
**      put Cm_regs  at:  CNAF (0,29,0,0)
*/	
#define CCSR            &(Cm_regs->ccsr)
#define CCSR_DEF        0x082c
#define CCSR_ERR        0x2bad

#define SLOT_Z          28      /* slot in all crates   */
#define SLOT_CES        29
#define SLOT_CTRL       30      /* slot in all crates   */

/* 
** VME CBD 8210  CAMAC status macro's.
*/
#define	Q		0x8000	/* Q-response received		*/
#define	X		0x4000	/* X-response received		*/
#define	CAM_TO		0x2000	/* time-out signalled		*/
#define	CAM_BD		0x1000	/* BD read transparent status	*/
#define	MAXI		0x0800	/* mask NO_X interrupt bit	*/
#define	MINTO		0x0020	/* mask internal time-out bit	*/
#define	MLAM		0x0010	/* mask LAM interrupts		*/
#define	MINT4		0x0004	/* mask external interrupt lev 4 */
#define	MINT2		0x0002	/* mask external interrupt lev 2 */

#define	Q_RESPONSE(status)	(  (status) & Q)
#define	NO_Q_RESPONSE(status)	(!((status) & Q))
#define	X_RESPONSE(status)	(  (status) & X)
#define	NO_X_RESPONSE(status)	(!((status) & X))

#define SWDES                   2
/* physical address for a given CAMAC slot */
#ifdef	linux
#define HARD_ADDR(N)            (((N) >> 2) * VME_PAGESIZE * PAGES_PER_4_SLOTS + CM_VME_BASE)
#endif
#ifndef linux
#define HARD_ADDR(N)            (((N) >> 2) * MMU_PAGESIZE * PAGES_PER_4_SLOTS + CM_VME_BASE)
#endif
#define NAFA(N,A,F)             (cm_s_regs->so_ptr + ((A) << 7) + ((F) << 2))
#define	UNAF(N,A,F)		((struct usword *)(NAFA(N,A,F) + SWDES))

#define NO_PANTS(D)             (((D) >> 16) & 0xFFFF)
#define TOPLESS(D)              (((D)      ) & 0xFFFF)

/*
** MAIN_LEVEL is level on which level 3 VME is maintained by the system bus
** it is defined in vme_conf.h and used to preset VME on SPARC's
** Should be equal to SBS_INTR_LEVEL on Sparc 4c/4m arch.
*/
/****** hardware dependencies ****************************************/
#ifdef	sun4e
#define         MAIN_LEVEL      5
#endif
#ifdef	sun4c
#define         MAIN_LEVEL      SBS_INTR_LEVEL
#endif	
#ifdef	sun4m
#define         MAIN_LEVEL      SBS_INTR_LEVEL
#endif	
/*********************************************************************/

/*
** Device oriented macro's
*/
#define  SLOT(node)              MINOR(node->i_rdev)
#define  N_EXT(n,c)             ((n) + ((c) << 5))
#define  N_CR(N)                ((N) & 0x1f)
#define  CRATE(cminor)          (((cminor) >> 5) & 0x3)

/* Interrupts */
#define	_amd_intr_lev(N)	(((N_CR(N)) - 1) % 8)
#define	_amd_intr_cmd(N)	WR_MEM(CAMVECLEN,_amd_intr_lev(N))
#define	_amd_pos(N)		(((N_CR(N)) - 1)/8)

#define	SLOT_CLOSED		0
#define	SLOT_OPENED		2
#endif	CM_H
